module ActiveRecord
  module Blakstar
    module Like
      def self.included(base)
        base.extend(ClassMethods)
      end
    
      module ClassMethods
        def blakstar_like(options = {})
          has_many :likes, :as => :likeable, :dependent => :destroy
          
          include ActiveRecord::Blakstar::Like::InstanceMethods
        end
      end
      
      module InstanceMethods
        # Likes the object by a given value. A user object can be passed to the method.
        def like!( user )
          return unless user.class.to_s == 'User'
          ActiveRecord::Base.transaction do
            like = likes.create( :value => 1, :user_id => user.id )
            self.increment!( :likes_count ) if !like.changed? && self.respond_to?( :likes_count )
            like
          end
        end
        
        def dislike!( user )
          return unless user.class.to_s == 'User'
          ActiveRecord::Base.transaction do
            like = likes.create( :value => -1, :user_id => user.id )
            self.increment!( :dislikes_count ) if !like.changed? && self.respond_to?( :dislikes_count )
            like
          end
        end
        
        # Calculates the average rating. Calculation based on the already given values.
        def average_like
          return 0 if likes.empty?
          count_likes! + count_dislikes!
        end
        
        def count_likes!
          self.respond_to?( :likes_count) ? self.likes_count : likes.where('value = 1').count
        end
        
        def count_dislikes!
          self.respond_to?( :dislikes_count) ? self.dislikes_count : likes.where('value = -1').count
        end
        
        # Checks wheter a user liked or disliked the object or not.
        def liked_by?( user )
          return false if user.nil? || user.class.to_s != 'User'
          likes.detect {|l| l.user_id == user.id }
        end
      end
      
    end
  end
end